@@ -30,7 +30,7 @@
  *            Sven Neumann <sven@convergence.de>
  */
 
-#include "config.h"
+#include <config.h>
 #include "gdk.h"
 
 #include <stdlib.h>
@@ -1629,7 +1629,28 @@ gdk_directfb_translate_key (DFBInputDeviceKeyIdentifier key_id,
 				break;
 
 			case DIKT_DEAD:
-				/* dead keys are handled directly by directfb */
+				switch (key_symbol)
+				{
+					case DIKS_DEAD_ABOVEDOT:     keyval = GDK_dead_abovedot;     break;
+					case DIKS_DEAD_ABOVERING:    keyval = GDK_dead_abovering;    break;
+					case DIKS_DEAD_ACUTE:        keyval = GDK_dead_acute;        break;
+					case DIKS_DEAD_BREVE:        keyval = GDK_dead_breve;        break;
+					case DIKS_DEAD_CARON:        keyval = GDK_dead_caron;        break;
+					case DIKS_DEAD_CEDILLA:      keyval = GDK_dead_cedilla;      break;
+					case DIKS_DEAD_CIRCUMFLEX:   keyval = GDK_dead_circumflex;   break;
+					case DIKS_DEAD_DIAERESIS:    keyval = GDK_dead_diaeresis;    break;
+					case DIKS_DEAD_DOUBLEACUTE:  keyval = GDK_dead_doubleacute;  break;
+					case DIKS_DEAD_GRAVE:        keyval = GDK_dead_grave;        break;
+					case DIKS_DEAD_IOTA:         keyval = GDK_dead_iota;         break;
+					case DIKS_DEAD_MACRON:       keyval = GDK_dead_macron;       break;
+					case DIKS_DEAD_OGONEK:       keyval = GDK_dead_ogonek;       break;
+					case DIKS_DEAD_SEMIVOICED_SOUND:
+								     keyval = GDK_dead_semivoiced_sound;                        break;
+					case DIKS_DEAD_TILDE:        keyval = GDK_dead_tilde;        break;
+					case DIKS_DEAD_VOICED_SOUND: keyval = GDK_dead_voiced_sound; break;
+					default:
+								     break;
+				}
 				break;
 
 			case DIKT_CUSTOM:
@@ -1694,7 +1715,7 @@ _gdk_directfb_keyboard_exit (void)
 
 void
 gdk_directfb_translate_key_event (DFBWindowEvent *dfb_event,
-                                  GdkEventKey    *event)
+		GdkEventKey    *event)
 {
 	gint  len;
 	gchar buf[6];
@@ -1732,32 +1753,6 @@ gdk_directfb_translate_key_event (DFBWindowEvent *dfb_event,
 	event->length = len;
 }
 
-/**
- * gdk_keymap_get_caps_lock_state:
- * @keymap: a #GdkKeymap
- *
- * Returns whether the Caps Lock modifer is locked.
- *
- * Returns: %TRUE if Caps Lock is on
- *
- * Since: 2.16
- */
-gboolean
-gdk_keymap_get_caps_lock_state (GdkKeymap *keymap)
-{
-  IDirectFBInputDevice *keyboard = _gdk_display->keyboard;
-
-  if (keyboard)
-    {
-      DFBInputDeviceLockState  state;
-
-      if (keyboard->GetLockState (keyboard, &state) == DFB_OK)
-        return ((state & DILS_CAPS) != 0);
-    }
-
-  return FALSE;
-}
-
 /**
  * gdk_keymap_get_entries_for_keycode:
  * @keymap: a #GdkKeymap or %NULL to use the default keymap
@@ -1903,6 +1898,30 @@ gdk_keymap_get_entries_for_keyval (GdkKeymap     *keymap,
 	return (*n_keys > 0);
 }
 
+/**
+ * gdk_keymap_translate_keyboard_state:
+ * @keymap: a #GdkKeymap, or %NULL to use the default
+ * @keycode: a hardware keycode
+ * @state: a modifier state 
+ * @group: active keyboard group
+ * @keyval: return location for keyval
+ * @effective_group: return location for effective group
+ * @level: return location for level
+ * @consumed_modifiers: return location for modifiers that were used to 
+ *                      determine the group or level
+ *
+ * Translates the contents of a #GdkEventKey into a keyval, effective
+ * group, and level. Modifiers that affected the translation and
+ * are thus unavailable for application use are returned in
+ * @consumed_modifiers.  See gdk_keyval_get_keys() for an explanation of
+ * groups and levels.  The @effective_group is the group that was
+ * actually used for the translation; some keys such as Enter are not
+ * affected by the active keyboard group. The @level is derived from
+ * @state. For convenience, #GdkEventKey already contains the translated
+ * keyval, so this function isn't as useful as you might think.
+ * 
+ * Return value: %TRUE if there was a keyval bound to the keycode/state/group
+ **/
 gboolean
 gdk_keymap_translate_keyboard_state (GdkKeymap       *keymap,
 		guint            keycode,
@@ -1922,41 +1941,32 @@ gdk_keymap_translate_keyboard_state (GdkKeymap       *keymap,
 
 		if (directfb_keymap[index + i + 2 * group] != GDK_VoidSymbol)
 		{
-			if (keyval)
-				*keyval = directfb_keymap[index + i + 2 * group];
+			*keyval = directfb_keymap[index + i + 2 * group];
 
 			if (group && directfb_keymap[index + i] == *keyval)
 			{
-				if (effective_group)
-					*effective_group = 0;
-                if(consumed_modifiers)
-				    *consumed_modifiers = 0;
+				*effective_group = 0;
+				*consumed_modifiers = 0;
 			}
 			else
 			{
-				if (effective_group)
-					*effective_group = group;
-                if(consumed_modifiers)
-				    *consumed_modifiers = GDK_MOD2_MASK;
+				*effective_group = group;
+				*consumed_modifiers = GDK_MOD2_MASK;
 			}
-			if (level)
-				*level = i;
+
+			*level = i;
 
 			if (i && directfb_keymap[index + 2 * *effective_group] != *keyval)
-                if(consumed_modifiers)
-				    *consumed_modifiers |= GDK_SHIFT_MASK;
+				*consumed_modifiers |= GDK_SHIFT_MASK;
 
 			return TRUE;
 		}
 	}
-	if (keyval)
-		*keyval             = 0;
-	if (effective_group)
-		*effective_group    = 0;
-	if (level)
-		*level              = 0;
-    if(consumed_modifiers)
-	    *consumed_modifiers = 0;
+
+	*keyval             = 0;
+	*effective_group    = 0;
+	*level              = 0;
+	*consumed_modifiers = 0;
 
 	return FALSE;
 }
@@ -1998,21 +2008,6 @@ gdk_keymap_lookup_key (GdkKeymap          *keymap,
 	return 0;
 }
 
-void
-gdk_keymap_add_virtual_modifiers (GdkKeymap       *keymap,
-                                  GdkModifierType *state)
-{
-	g_warning ("gdk_keymap_add_virtual_modifiers unimplemented \n");
-}
-
-gboolean
-gdk_keymap_map_virtual_modifiers (GdkKeymap       *keymap,
-                                  GdkModifierType *state)
-{
-  g_warning ("gdk_keymap_map_virtual_modifiers unimplemented \n");
-
-  return TRUE;
-}
 
 #define __GDK_KEYS_DIRECTFB_C__
 #include "gdkaliasdef.c"
