@@ -1,5 +1,5 @@
 /* GDK - The GIMP Drawing Kit
- * Copyright (C) 2002 Hans Breuer
+ * Copyright (C) 2002,2005 Hans Breuer
  * Copyright (C) 2003 Tor Lillqvist
  *
  * This library is free software; you can redistribute it and/or
@@ -148,7 +148,7 @@ _gdk_monitor_init (void)
 	{
 	  _gdk_monitors[i].x += _gdk_offset_x;
 	  _gdk_monitors[i].y += _gdk_offset_y;
-	  GDK_NOTE (MISC, g_print ("Monitor %d: %dx%d@+%d+%d\n",
+	  GDK_NOTE (MISC, g_print ("Monitor %d: %dx%d@%+d%+d\n",
 				   i, _gdk_monitors[i].width,
 				   _gdk_monitors[i].height,
 				   _gdk_monitors[i].x, _gdk_monitors[i].y));
@@ -175,11 +175,53 @@ _gdk_monitor_init (void)
 
 }
 
+/*
+ * Dynamic version of ProcessIdToSessionId() form Terminal Service.
+ * It is only returning something else than 0 when running under
+ * Terminal Service, available since NT4 SP4 and not for win9x
+ */
+static guint
+get_session_id (void)
+{
+  typedef BOOL (WINAPI *t_ProcessIdToSessionId) (DWORD, DWORD*);
+  static t_ProcessIdToSessionId p_ProcessIdToSessionId = NULL;
+  static HMODULE kernel32 = NULL;
+  DWORD id = 0;
+
+  if (kernel32 == NULL)
+    {
+      kernel32 = GetModuleHandle ("kernel32.dll");
+
+      g_assert (kernel32 != NULL);
+
+      p_ProcessIdToSessionId = (t_ProcessIdToSessionId) GetProcAddress (kernel32, "ProcessIdToSessionId");
+   }
+  if (p_ProcessIdToSessionId)
+      p_ProcessIdToSessionId (GetCurrentProcessId (), &id); /* got it (or not ;) */
+
+  return id;
+}
+
 GdkDisplay *
 gdk_display_open (const gchar *display_name)
 {
-  if (_gdk_display != NULL)
-    return NULL; /* single display only */
+  GDK_NOTE (MISC, g_print ("gdk_display_open: %s\n", (display_name ? display_name : "NULL")));
+
+  if (display_name == NULL ||
+      g_ascii_strcasecmp (display_name,
+			  gdk_display_get_name (_gdk_display)) == 0)
+    {
+      if (_gdk_display != NULL)
+	{
+	  GDK_NOTE (MISC, g_print ("... return _gdk_display\n"));
+	  return _gdk_display;
+	}
+    }
+  else
+    {
+      GDK_NOTE (MISC, g_print ("... return NULL\n"));
+      return NULL;
+    }
 
   _gdk_display = g_object_new (GDK_TYPE_DISPLAY, NULL);
   _gdk_screen = g_object_new (GDK_TYPE_SCREEN, NULL);
@@ -194,21 +236,78 @@ gdk_display_open (const gchar *display_name)
   _gdk_input_init (_gdk_display);
   _gdk_dnd_init ();
 
+  /* Precalculate display name */
+  (void) gdk_display_get_name (_gdk_display);
+
   g_signal_emit_by_name (gdk_display_manager_get (),
 			 "display_opened", _gdk_display);
 
+  GDK_NOTE (MISC, g_print ("... _gdk_display now set up\n"));
+
   return _gdk_display;
 }
 
 G_CONST_RETURN gchar *
 gdk_display_get_name (GdkDisplay *display)
 {
-  return gdk_get_display_arg_name ();
+  HDESK hdesk = GetThreadDesktop (GetCurrentThreadId ());
+  char dummy;
+  char *desktop_name;
+  HWINSTA hwinsta = GetProcessWindowStation ();
+  char *window_station_name;
+  DWORD n;
+  char *display_name;
+  static const char *display_name_cache = NULL;
+
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
+  
+  if (display_name_cache != NULL)
+    return display_name_cache;
+
+  n = 0;
+  GetUserObjectInformation (hdesk, UOI_NAME, &dummy, 0, &n);
+  if (n == 0)
+    desktop_name = "Default";
+  else
+    {
+      n++;
+      desktop_name = g_alloca (n + 1);
+      memset (desktop_name, 0, n + 1);
+
+      if (!GetUserObjectInformation (hdesk, UOI_NAME, desktop_name, n, &n))
+	desktop_name = "Default";
+    }
+
+  n = 0;
+  GetUserObjectInformation (hwinsta, UOI_NAME, &dummy, 0, &n);
+  if (n == 0)
+    window_station_name = "WinSta0";
+  else
+    {
+      n++;
+      window_station_name = g_alloca (n + 1);
+      memset (window_station_name, 0, n + 1);
+
+      if (!GetUserObjectInformation (hwinsta, UOI_NAME, window_station_name, n, &n))
+	window_station_name = "WinSta0";
+    }
+
+  display_name = g_strdup_printf ("%d\\%s\\%s",
+				  get_session_id (), window_station_name,
+				  desktop_name);
+
+  GDK_NOTE (MISC, g_print ("gdk_display_get_name: %s\n", display_name));
+
+  display_name_cache = display_name;
+
+  return display_name_cache;
 }
 
 gint
 gdk_display_get_n_screens (GdkDisplay *display)
 {
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), 0);
+  
   return 1;
 }
 
@@ -216,12 +315,17 @@ GdkScreen *
 gdk_display_get_screen (GdkDisplay *display,
 			gint        screen_num)
 {
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
+  g_return_val_if_fail (screen_num == 0, NULL);
+
   return _gdk_screen;
 }
 
 GdkScreen *
 gdk_display_get_default_screen (GdkDisplay *display)
 {
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
+
   return _gdk_screen;
 }
 
@@ -240,134 +344,15 @@ gdk_display_supports_selection_notification (GdkDisplay *display)
 {
   g_return_val_if_fail (GDK_IS_DISPLAY (display), FALSE);
 
-  return TRUE;
+  return FALSE;
 }
 
-static HWND _hwnd_next_viewer = NULL;
-
-/*
- * maybe this should be integrated with the default message loop - or maybe not ;-)
- */
-static LRESULT CALLBACK
-_win32_on_clipboard_change (HWND   hwnd,
-                            UINT   message,
-                            WPARAM wparam,
-                            LPARAM lparam)
-{
-  switch (message)
-    {
-    case WM_DESTROY : /* remove us from chain */
-      {
-        ChangeClipboardChain (hwnd, _hwnd_next_viewer);
-        return 0; 
-      }
-    case WM_CHANGECBCHAIN :
-      {
-        HWND hwndRemove = (HWND) wparam; /* handle of window being removed */
-        HWND hwndNext   = (HWND) lparam; /* handle of next window in chain */
-        if (hwndRemove == _hwnd_next_viewer)
-          _hwnd_next_viewer = hwndNext == hwnd ? NULL : hwndNext;
-        return 0;
-      }
-    case WM_DRAWCLIPBOARD :
-      {
-        /* create the appropriate gdk events */
-        HWND hwndOwner = GetClipboardOwner ();
-        UINT nFormat = 0;
-        int n = 0;
-
-        if (OpenClipboard (hwnd))
-          { 
-            for (; 0 != (nFormat = EnumClipboardFormats (nFormat)); )
-              {
-                char sFormat[80];
-                if (GetClipboardFormatName (nFormat, sFormat, 80) > 0)
-                  g_print ("%s ", sFormat);
-                n++; /* do something useful ? */
-              }
-            GDK_NOTE (DND, 
-                      g_print ("WM_DRAWCLIPBOARD :  formats %d owner %#lx\n", n, hwndOwner));
-
-            CloseClipboard ();
-          }
-        /* XXX: generate the apropriate GdkEventOwnerChange ... */
-
-        /* don't break the chain */
-        return PostMessage (_hwnd_next_viewer, message, wparam, lparam);
-      }
-    default :
-      return DefWindowProc (hwnd, message, wparam, lparam);
-    }
-}
-
-/*
- * Creates a hidden window and adds it to the clipboard chain
- */
-HWND
-_gdk_win32_register_clipboard_notification (void)
-{
-  WNDCLASS wclass;
-  HWND     hwnd;
-  ATOM     klass;
-
-  memset (&wclass, 0, sizeof(WNDCLASS));
-  wclass.lpszClassName = "GdkClipboardNotification";
-  wclass.lpfnWndProc   = _win32_on_clipboard_change;
-  wclass.hInstance     = _gdk_app_hmodule;
-
-  klass = RegisterClass (&wclass);
-  if (!klass)
-    return NULL;
-
-  hwnd = CreateWindow (MAKEINTRESOURCE(klass),
-                       NULL, WS_POPUP,
-                       0, 0, 0, 0, NULL, NULL,
-                       _gdk_app_hmodule, NULL);
-  if (!hwnd)
-    {
-      UnregisterClass (MAKEINTRESOURCE(klass), _gdk_app_hmodule);
-      return NULL;
-    }
-  _hwnd_next_viewer = SetClipboardViewer (hwnd);
-  return hwnd;
-}
-
-/*
- * The whole function would only make sense if the gdk/win32 clipboard
- * model is rewritten to do delayed rendering. Currently this is only
- * testcode and as noted in
- * http://mail.gnome.org/archives/gtk-devel-list/2004-May/msg00113.html
- * probably not worth bothering ;)
- */
 gboolean 
 gdk_display_request_selection_notification (GdkDisplay *display,
                                             GdkAtom     selection)
 
 {
-  static HWND hwndViewer = NULL;
-  gboolean ret = FALSE;
-
-  GDK_NOTE (DND, 
-            g_print ("gdk_display_request_selection_notification (..., %s)",
-                     gdk_atom_name (selection)));
-
-  if (GDK_SELECTION_CLIPBOARD == selection)
-    {
-      if (!hwndViewer)
-        {
-          hwndViewer = _gdk_win32_register_clipboard_notification ();
-          GDK_NOTE (DND, g_print (" registered"));
-        }
-      ret = (hwndViewer != NULL);
-    }
-  else if (GDK_SELECTION_PRIMARY == selection)
-    {
-      /* seems to work by default ? */
-      GDK_NOTE (DND, g_print (" by default"));
-      ret = TRUE;
-    }
-  GDK_NOTE (DND, g_print (" -> %s\n", ret ? "TRUE" : "FALSE"));
-  return ret;
+  return FALSE;
 }
 
 gboolean
@@ -383,5 +368,24 @@ gdk_display_store_clipboard (GdkDisplay *display,
 			     GdkAtom    *targets,
 			     gint        n_targets)
 {
-  /* XXX: implement it (or maybe not as long as we don't support delayed rendering?) */
+}
+
+gboolean 
+gdk_display_supports_shapes (GdkDisplay *display)
+{
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), FALSE);
+
+  return TRUE;
+}
+
+gboolean 
+gdk_display_supports_input_shapes (GdkDisplay *display)
+{
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), FALSE);
+
+  /* Not yet implemented. See comment in
+   * gdk_window_input_shape_combine_mask().
+   */
+
+  return FALSE;
 }
