@@ -24,7 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include "config.h"
+#include <config.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 
@@ -70,7 +70,7 @@ gdk_visual_get_type (void)
 
   if (!object_type)
     {
-      const GTypeInfo object_info =
+      static const GTypeInfo object_info =
       {
         sizeof (GdkVisualClass),
         (GBaseInitFunc) NULL,
@@ -95,7 +95,7 @@ gdk_visual_get_type (void)
 void
 _gdk_visual_init (GdkScreen *screen)
 {
-  static const gint possible_depths[8] = { 32, 30, 24, 16, 15, 8, 4, 1 };
+  static const gint possible_depths[7] = { 32, 24, 16, 15, 8, 4, 1 };
   static const GdkVisualType possible_types[6] =
     {
       GDK_VISUAL_DIRECT_COLOR,
@@ -119,7 +119,6 @@ _gdk_visual_init (GdkScreen *screen)
   g_return_if_fail (GDK_IS_SCREEN (screen));
   screen_x11 = GDK_SCREEN_X11 (screen);
 
-  nxvisuals = 0;
   visual_template.screen = screen_x11->screen_num;
   visual_list = XGetVisualInfo (screen_x11->xdisplay, VisualScreenMask, &visual_template, &nxvisuals);
   
@@ -349,7 +348,7 @@ gdk_visual_get_best_type (void)
  * This is the visual for the root window of the display.
  * The return value should not be freed.
  * 
- * Return value: (transfer none): the system visual
+ * Return value: the system visual
  *
  * Since: 2.2
  **/
@@ -367,7 +366,7 @@ gdk_screen_get_system_visual (GdkScreen * screen)
  * Get the visual with the most available colors for the default
  * GDK screen. The return value should not be freed.
  * 
- * Return value: (transfer none): best visual
+ * Return value: best visual
  **/
 GdkVisual*
 gdk_visual_get_best (void)
@@ -386,7 +385,7 @@ gdk_visual_get_best (void)
  * over grayscale or fixed-colormap visuals. The return value should not
  * be freed. %NULL may be returned if no visual supports @depth.
  * 
- * Return value: (transfer none): best visual for the given depth
+ * Return value: best visual for the given depth
  **/
 GdkVisual*
 gdk_visual_get_best_with_depth (gint depth)
@@ -415,7 +414,7 @@ gdk_visual_get_best_with_depth (gint depth)
  * should not be freed. %NULL may be returned if no visual has type
  * @visual_type.
  * 
- * Return value: (transfer none): best visual of the given type
+ * Return value: best visual of the given type
  **/
 GdkVisual*
 gdk_visual_get_best_with_type (GdkVisualType visual_type)
@@ -442,8 +441,7 @@ gdk_visual_get_best_with_type (GdkVisualType visual_type)
  *
  * Combines gdk_visual_get_best_with_depth() and gdk_visual_get_best_with_type().
  * 
- * Return value: (transfer none): best visual with both @depth and
- *     @visual_type, or %NULL if none
+ * Return value: best visual with both @depth and @visual_type, or %NULL if none
  **/
 GdkVisual*
 gdk_visual_get_best_with_both (gint          depth,
@@ -467,8 +465,8 @@ gdk_visual_get_best_with_both (gint          depth,
 
 /**
  * gdk_query_depths:
- * @depths: (out) (array): return location for available depths
- * @count: (out): return location for number of available depths
+ * @depths: return location for available depths 
+ * @count: return location for number of available depths
  *
  * This function returns the available bit depths for the default
  * screen. It's equivalent to listing the visuals
@@ -661,7 +659,7 @@ gdk_x11_visual_get_xvisual (GdkVisual *visual)
  * 
  * Gets the screen to which this visual belongs
  * 
- * Return value: (transfer none): the screen to which this visual belongs.
+ * Return value: the screen to which this visual belongs.
  *
  * Since: 2.2
  **/
