@@ -31,11 +31,11 @@
 #include <string.h>
 #include <stdarg.h>
 #include <gdkkeysyms.h>
-#include "gtkalias.h"
 #include "gtkbindings.h"
 #include "gtkkeyhash.h"
 #include "gtkwidget.h"
 #include "gtkrc.h"
+#include "gtkalias.h"
 
 
 /* --- defines --- */
@@ -44,9 +44,11 @@
 
 /* --- structures --- */
 typedef struct {
+  GtkPathType   type;
   GPatternSpec *pspec;
-  gpointer user_data;
-  guint seq_id;
+  GSList       *path;
+  gpointer      user_data;
+  guint         seq_id;
 } PatternSpec;
 
 
@@ -54,22 +56,31 @@ typedef struct {
 static GHashTable	*binding_entry_hash_table = NULL;
 static GSList           *binding_key_hashes = NULL;
 static GSList		*binding_set_list = NULL;
-static const gchar	*key_class_binding_set = "gtk-class-binding-set";
+static const gchar	 key_class_binding_set[] = "gtk-class-binding-set";
 static GQuark		 key_id_class_binding_set = 0;
 
 
 /* --- functions --- */
+static void
+pattern_spec_free (PatternSpec *pspec)
+{
+  _gtk_rc_free_widget_class_path (pspec->path);
+  if (pspec->pspec)
+    g_pattern_spec_free (pspec->pspec);
+  g_free (pspec);
+}
+
 static GtkBindingSignal*
 binding_signal_new (const gchar *signal_name,
 		    guint	 n_args)
 {
   GtkBindingSignal *signal;
-  
-  signal = g_new (GtkBindingSignal, 1);
+
+  signal = (GtkBindingSignal *) g_slice_alloc0 (sizeof (GtkBindingSignal) + n_args * sizeof (GtkBindingArg));
   signal->next = NULL;
-  signal->signal_name = g_strdup (signal_name);
+  signal->signal_name = (gchar *)g_intern_string (signal_name);
   signal->n_args = n_args;
-  signal->args = g_new0 (GtkBindingArg, n_args);
+  signal->args = (GtkBindingArg *)(signal + 1);
   
   return signal;
 }
@@ -84,9 +95,7 @@ binding_signal_free (GtkBindingSignal *sig)
       if (G_TYPE_FUNDAMENTAL (sig->args[i].arg_type) == G_TYPE_STRING)
 	g_free (sig->args[i].d.string_data);
     }
-  g_free (sig->args);
-  g_free (sig->signal_name);
-  g_free (sig);
+  g_slice_free1 (sizeof (GtkBindingSignal) + sig->n_args * sizeof (GtkBindingArg), sig);
 }
 
 static guint
@@ -559,7 +568,7 @@ gtk_binding_set_new (const gchar    *set_name)
   g_return_val_if_fail (set_name != NULL, NULL);
   
   binding_set = g_new (GtkBindingSet, 1);
-  binding_set->set_name = g_strdup (set_name);
+  binding_set->set_name = g_intern_string (set_name);
   binding_set->widget_path_pspecs = NULL;
   binding_set->widget_class_pspecs = NULL;
   binding_set->class_branch_pspecs = NULL;
@@ -772,7 +781,7 @@ gtk_binding_entry_add_signal (GtkBindingSet  *binding_set,
     {
       GtkBindingArg *arg;
 
-      arg = g_new0 (GtkBindingArg, 1);
+      arg = g_slice_new0 (GtkBindingArg);
       slist = g_slist_prepend (slist, arg);
 
       arg->arg_type = va_arg (args, GtkType);
@@ -831,7 +840,7 @@ gtk_binding_entry_add_signal (GtkBindingSet  *binding_set,
   free_slist = slist;
   while (slist)
     {
-      g_free (slist->data);
+      g_slice_free (GtkBindingArg, slist->data);
       slist = slist->next;
     }
   g_slist_free (free_slist);
@@ -871,7 +880,18 @@ gtk_binding_set_add_path (GtkBindingSet	     *binding_set,
     }
   
   pspec = g_new (PatternSpec, 1);
-  pspec->pspec = g_pattern_spec_new (path_pattern);
+  pspec->type = path_type;
+  if (path_type == GTK_PATH_WIDGET_CLASS)
+    {
+      pspec->pspec = NULL;
+      pspec->path = _gtk_rc_parse_widget_class_path (path_pattern);
+    }
+  else
+    {
+      pspec->pspec = g_pattern_spec_new (path_pattern);
+      pspec->path = NULL;
+    }
+    
   pspec->seq_id = priority << 28;
   pspec->user_data = binding_set;
   
@@ -887,8 +907,7 @@ gtk_binding_set_add_path (GtkBindingSet	     *binding_set,
 	{
 	  GtkPathPriorityType lprio = tmp_pspec->seq_id >> 28;
 
-	  g_pattern_spec_free (pspec->pspec);
-	  g_free (pspec);
+	  pattern_spec_free (pspec);
 	  pspec = NULL;
 	  if (lprio < priority)
 	    {
@@ -909,25 +928,32 @@ static gboolean
 binding_match_activate (GSList          *pspec_list,
 			GtkObject	*object,
 			guint	         path_length,
-			const gchar     *path,
-			const gchar     *path_reversed)
+			gchar           *path,
+			gchar           *path_reversed)
 {
   GSList *slist;
 
   for (slist = pspec_list; slist; slist = slist->next)
     {
       PatternSpec *pspec;
+      GtkBindingSet *binding_set;
 
+      binding_set = NULL;
       pspec = slist->data;
-      if (g_pattern_match (pspec->pspec, path_length, path, path_reversed))
-	{
-	  GtkBindingSet *binding_set;
+      
+      if (pspec->type == GTK_PATH_WIDGET_CLASS)
+        {
+          if (_gtk_rc_match_widget_class (pspec->path, path_length, path, path_reversed))
+	    binding_set = pspec->user_data;
+        }
+      else
+        {
+          if (g_pattern_match (pspec->pspec, path_length, path, path_reversed))
+	    binding_set = pspec->user_data;
+        }
 
-	  binding_set = pspec->user_data;
-
-	  if (gtk_binding_entry_activate (binding_set->current, object))
-	    return TRUE;
-	}
+      if (binding_set && gtk_binding_entry_activate (binding_set->current, object))
+        return TRUE;
     }
 
   return FALSE;
@@ -1054,14 +1080,15 @@ gtk_bindings_activate_list (GtkObject *object,
       while (class_type && !handled)
 	{
 	  guint path_length;
-	  const gchar *path;
+	  gchar *path;
 	  gchar *path_reversed;
 	  
-	  path = g_type_name (class_type);
+	  path = g_strdup (g_type_name (class_type));
 	  path_reversed = g_strdup (path);
 	  g_strreverse (path_reversed);
 	  path_length = strlen (path);
 	  handled = binding_match_activate (patterns, object, path_length, path, path_reversed);
+	  g_free (path);
 	  g_free (path_reversed);
 
 	  class_type = g_type_parent (class_type);
@@ -1419,8 +1446,7 @@ free_pattern_specs (GSList *pattern_specs)
 
       pspec = slist->data;
 
-      g_pattern_spec_free (pspec->pspec);
-      g_free (pspec);
+      pattern_spec_free (pspec);
     }
 
   g_slist_free (pattern_specs);
@@ -1443,7 +1469,6 @@ binding_set_delete (GtkBindingSet *binding_set)
   free_pattern_specs (binding_set->widget_class_pspecs);
   free_pattern_specs (binding_set->class_branch_pspecs);
 
-  g_free (binding_set->set_name);
   g_free (binding_set);
 }
 
@@ -1504,3 +1529,6 @@ _gtk_binding_signal_new (const gchar	    *signal_name,
  
   return signal_id;
 }
+
+#define __GTK_BINDINGS_C__
+#include "gtkaliasdef.c"
