@@ -21,9 +21,12 @@
  * Authors: Alexander Larsson <alexl@redhat.com>
  */
 
+#include <config.h>
 #include "gtkprivate.h"
 #include "gtkwindow.h"
 #include "gtkmain.h"
+#include "gtkwindow-decorate.h"
+#include "gtkalias.h"
 
 
 #ifdef GDK_WINDOWING_FB
@@ -75,14 +78,14 @@ struct _GtkWindowDecoration
 
   GtkWindowResizeType resize;
   
-  gboolean moving : 1;
-  gboolean closing : 1;
-  gboolean maximizing : 1;
-  gboolean maximized : 1;
-  gboolean maximizable : 1;
-  gboolean decorated : 1;
-  gboolean real_inner_move : 1;
-  gboolean focused : 1;
+  guint moving : 1;
+  guint closing : 1;
+  guint maximizing : 1;
+  guint maximized : 1;
+  guint maximizable : 1;
+  guint decorated : 1;
+  guint real_inner_move : 1;
+  guint focused : 1;
 };
 
 #define DECORATION_BORDER_TOP 15
@@ -145,7 +148,7 @@ gtk_decorated_window_init (GtkWindow   *window)
   deco->maximizable = FALSE;
   deco->real_inner_move = FALSE;
  
-  g_object_set_data_full (G_OBJECT (window), "gtk-window-decoration", deco,
+  g_object_set_data_full (G_OBJECT (window), I_("gtk-window-decoration"), deco,
 			  (GDestroyNotify) gtk_decoration_free);
   
   gtk_window_set_has_frame (window, TRUE);
@@ -597,7 +600,7 @@ gtk_decorated_window_paint (GtkWidget    *widget,
       gint width, height;
 
       frame = window->frame;
-      gdk_window_get_size (frame, &width, &height);
+      gdk_drawable_get_size (frame, &width, &height);
 
       /* Top */
       gtk_paint_flat_box (widget->style, frame, GTK_STATE_NORMAL,
@@ -828,4 +831,5 @@ gtk_decorated_window_move_resize_window (GtkWindow   *window,
 #endif
 
 
-
+#define __GTK_WINDOW_DECORATE_C__
+#include "gtkaliasdef.c"
